/*
 * convertd - tk commands to convert between pixels and centimeters.
 *
 * Copyright (C) 1999 Gordon D. Carrie
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA  02111-1307, USA.
 * 
 * Please address questions and bug reports to gcarri@mail.txcyber.com
 *
 **********************************************************************
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <tk.h>
#include "tkConvertD.h"

static int pxPerCm;		/* Pixels per centimeter */
static double cmPerPx;		/* Meters per pixel */
static int loaded;		/* Tell if package already loaded */

int
Tkconvertd_Init(Tcl_Interp *interp)
{
  /* Initialize convertd package */
  Tk_Window tkwin = Tk_MainWindow(interp);
  char *varName;	/* Name of Tcl global variable for conversion. */
                        /* Must be in writable memory */

  if (loaded)
    return TCL_OK;
  loaded = 1;

  /* Set global conversion factors for pixels<->cm */
  if (Tk_GetPixels(interp, tkwin, "1.0 c", &pxPerCm) == TCL_ERROR) {
    Tcl_AppendResult(interp, "Could not get cm to pixel conversion", NULL);
    return TCL_ERROR;
  }
  cmPerPx = 1.0 / pxPerCm;

  /* Tcl global variables for pixel<->cm conversion */
  if ( !(varName = (char*)malloc(8 * sizeof(char))) ) {
    Tcl_AppendResult(interp, "Allocation failure in TkConvertd_Init", NULL);
    return TCL_ERROR;
  }
  if (Tcl_LinkVar(interp, strcpy(varName, "PxPerCm"), (char*)&pxPerCm,
                  TCL_LINK_INT) == TCL_ERROR)
    return TCL_ERROR;
  if (Tcl_LinkVar(interp, strcpy(varName, "CmPerPx"), (char*)&cmPerPx,
                  TCL_LINK_DOUBLE) == TCL_ERROR)
    return TCL_ERROR;

  Tcl_PkgProvide(interp, "convertd", "1.2");
  return TCL_OK;
}

/* Return conversion factors initializing them if necessary */
int PxPerCm(Tcl_Interp *interp) {
  if ( !pxPerCm ) {
    /* Initialize if necessary */
    Tk_Window tkwin = Tk_MainWindow(interp);
    if (Tk_GetPixels(interp, tkwin, "1.0 c", &pxPerCm) == TCL_ERROR) {
      Tcl_AppendResult(interp, "Could not get cm to pixel conversion", NULL);
      return TCL_ERROR;
    }
  }
  return  pxPerCm;
}

double CmPerPx(Tcl_Interp *interp)
{
  if ( !cmPerPx ) {
    /* Initialize if necessary */
    Tk_Window tkwin = Tk_MainWindow(interp);
    double mmPerPx;
    if (Tk_GetScreenMM(interp, tkwin, "1", &mmPerPx) == TCL_ERROR) {
      Tcl_AppendResult(interp, "Could not get pixel to mm conversion", NULL);
      return TCL_ERROR;
    }
    cmPerPx = 0.1 * mmPerPx;
  }
  return cmPerPx;
}
